// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.apache.log4j.Logger;

import de.uka.ilkd.key.proof.ConstraintTableModel;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.proof.Proof;


public class UserConstraintView extends JPanel implements java.io.Serializable {

    /** Text fields for new equalities */
    protected JTextField  newEqLeftText, newEqRightText;
    protected JButton     newEqAddButton, delEqButton;

    /** Status messages */
    protected JLabel      statusLabel;
    protected JLabel      constraintSatLabel;

    /** Constraint to be edited */
    protected ConstraintTableModel    constraintTableModel    = null;
    protected ConstraintTableListener constraintTableListener = new ConstraintTableListener ();
    protected JTable                  constraintTable;

    protected KeYMediator mediator                      = null;

    /** Listener for proof changes */
    protected SelectionListener selectionListener       = new SelectionListener ();

    /** Listener for GUI changes */
    protected ConstraintViewGUIListener guiListener     = new ConstraintViewGUIListener ();

    public UserConstraintView () {

	layoutPane();
	setVisible ( true );

	setControlsActive ( true );
	printSatisfiability ();

    }


    public void setMediator ( KeYMediator p_mediator ) {
	if ( mediator != null )
	    unregisterAtMediator ();
	
	mediator = p_mediator;
	registerAtMediator ();

	setConstraintTableModel ( mediator.getUserConstraint () );
    }


    protected void setConstraintTableModel ( ConstraintTableModel p_model ) {
	if ( constraintTableModel != null )
	    unregisterAtModel ();

	constraintTableModel = p_model;

	if ( constraintTableModel != null ) {
	    constraintTable.setModel ( constraintTableModel );
	    registerAtModel ();
	}

	setControlsActive ( true );
	printSatisfiability ();
    }


    protected void registerAtMediator () {
	mediator.addKeYSelectionListener              ( selectionListener );
	mediator.addGUIListener                       ( guiListener );
    }


    protected void unregisterAtMediator () {
	mediator.removeGUIListener                    ( guiListener );
	mediator.removeKeYSelectionListener           ( selectionListener );
    }


    protected void registerAtModel () {
	constraintTableModel.addTableModelListener    ( constraintTableListener );
    }


    protected void unregisterAtModel () {
	constraintTableModel.removeTableModelListener ( constraintTableListener );
    }


    protected void setControlsActive ( boolean p ) {
	if ( p && constraintTableModel != null ) {
	    constraintTable.setEnabled ( true );
	    newEqLeftText  .setEnabled ( true );
	    newEqRightText .setEnabled ( true );
	    newEqAddButton .setEnabled ( true );
	    delEqButton    .setEnabled ( true );	    
	} else {
	    constraintTable.setEnabled ( false );
	    newEqLeftText  .setEnabled ( false );
	    newEqRightText .setEnabled ( false );
	    newEqAddButton .setEnabled ( false );
	    delEqButton    .setEnabled ( false );	    
	}
    }


    /** Build everything */    
    protected void layoutPane() {
	setLayout ( new GridBagLayout () );
	GridBagConstraints c = new GridBagConstraints ();
	
	c.fill       = GridBagConstraints.BOTH;
	c.gridx      = 0;
	c.gridy      = 0;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 1;
	add ( createConstraintPanel(), c );

	c.fill       = GridBagConstraints.BOTH;
	c.gridx      = 0;
	c.gridy      = 1;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 2, 0, 0, 0 );
	add ( createStatusPanel(), c);
    }


    protected JPanel createStatusPanel() {
	statusLabel=new JLabel(" ");	
	JPanel statusLine = new JPanel( new BorderLayout () );

	statusLine.add ( statusLabel,   BorderLayout.CENTER );
	statusLabel.setBorder ( new BevelBorder(BevelBorder.LOWERED) );

	return statusLine;
    }


    /** Main area with table, text fields and buttons for adding/removing constraints */
    protected JPanel createConstraintPanel() {
	JPanel             constraintPanel = new JPanel ( new GridBagLayout () );
	GridBagConstraints c               = new GridBagConstraints ();
	
	constraintTable              = new JTable ( constraintTableModel );
        constraintTable.setPreferredScrollableViewportSize(new Dimension(300, 300));
	JScrollPane pane = new JScrollPane ( constraintTable );

	constraintTable.getSelectionModel ()
	    .addListSelectionListener ( new ListSelectionListener () {
		    public void valueChanged(ListSelectionEvent p_e) {
			if ( p_e.getValueIsAdjusting () ) return;
			ListSelectionModel lsm = (ListSelectionModel)p_e.getSource();
			delEqButton.setEnabled ( !lsm.isSelectionEmpty () );
		    } } );
	constraintTable.setSelectionMode ( ListSelectionModel.SINGLE_INTERVAL_SELECTION );

	newEqLeftText  = new JTextField ();
	newEqRightText = new JTextField ();

	newEqAddButton = new JButton ( "Add" );
	delEqButton    = new JButton ( "Del" );

	newEqAddButton.addActionListener ( new ActionListener () {
		public void actionPerformed ( ActionEvent p_e ) {
		    if ( addRow ( newEqLeftText .getText (),
				  newEqRightText.getText () ) ) {
			newEqLeftText .setText ( "" );
			newEqRightText.setText ( "" );
		    }
		} } );

	delEqButton.setEnabled ( false );
	delEqButton.addActionListener ( new ActionListener () {
		public void actionPerformed ( ActionEvent p_e ) {
		    int i;
		    while ( ( i = constraintTable.getSelectedRow () ) != -1 )
			constraintTableModel.deleteRow ( i );
		    delEqButton.setEnabled ( false );
		} } );

	JLabel addLabel    = new JLabel ( "Add new Equality:" );
	
	constraintSatLabel = new JLabel ( "", SwingConstants.RIGHT );

	c.fill       = GridBagConstraints.BOTH;
	c.gridx      = 0;
	c.gridy      = 0;
	c.gridwidth  = 2;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 1;
	c.insets     = new Insets ( 0, 0, 3, 0 );
	constraintPanel.add ( pane, c );

	c.fill       = GridBagConstraints.HORIZONTAL;
	c.gridx      = 0;
	c.gridy      = 1;
	c.gridwidth  = 2;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 0, 0, 0, 0 );
	constraintPanel.add ( constraintSatLabel, c );

	c.fill       = GridBagConstraints.NONE;
	c.anchor     = GridBagConstraints.WEST;
	c.gridx      = 0;
	c.gridy      = 2;
	c.gridwidth  = 2;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 0, 0, 0, 0 );
	constraintPanel.add ( addLabel, c );

	c.anchor     = GridBagConstraints.CENTER;

	c.fill       = GridBagConstraints.HORIZONTAL;
	c.gridx      = 0;
	c.gridy      = 3;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 0, 0, 0, 4 );
	constraintPanel.add ( newEqLeftText, c );

	c.fill       = GridBagConstraints.HORIZONTAL;
	c.gridx      = 1;
	c.gridy      = 3;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 0, 4, 0, 0 );
	constraintPanel.add ( newEqRightText, c );

	c.fill       = GridBagConstraints.HORIZONTAL;
	c.gridx      = 0;
	c.gridy      = 4;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 6, 0, 0, 4 );
	constraintPanel.add ( newEqAddButton, c );

	c.fill       = GridBagConstraints.HORIZONTAL;
	c.gridx      = 1;
	c.gridy      = 4;
	c.gridwidth  = 1;
	c.gridheight = 1;
	c.weightx    = 1;
	c.weighty    = 0;
	c.insets     = new Insets ( 6, 4, 0, 0 );
	constraintPanel.add ( delEqButton, c );


	constraintPanel.setBorder( new CompoundBorder (
		   new TitledBorder("Constraint Equalities"),
		   new EmptyBorder ( 2, 2, 2, 2 ) ) );
	
	return constraintPanel;
    }


    /**
     * Will be called when this dialog will be closed
     */
    protected void closeDlg(){
	//	mediator.freeModalAccess(this); 	
    }


    
    protected void printStatus ( String p_status ) {
	statusLabel.setText ( p_status );
    }


    protected synchronized void printSatisfiability () {
	if ( constraintTableModel != null ) {
	    if ( constraintTableModel.getConstraint ().isSatisfiable () ) {
		constraintSatLabel.setText ( "Constraint is satisfiable" );
		constraintSatLabel.setForeground
		    ( UIManager.getColor ( "Label.foreground" ) );
	    } else {
		constraintSatLabel.setText ( "Constraint is not satisfiable" );
		constraintSatLabel.setForeground ( Color.red );
	    }
	} else
	    constraintSatLabel.setText ( "" );	    
    }


    /**
     * creates a new Termparser that parses the given string using
     * namespaces and services from the mediator
     * @param s the String to parse 
     */
    public Term parseTerm (String s) throws ParserException {        
        final SimpleTermParser parser = TermParserFactory.createInstance ();
        return parser.parse ( new StringReader ( s ),
                              null,
                              mediator.getServices (),
                              mediator.namespaces (),
                              mediator.getNotationInfo ().getAbbrevMap () );
    }

    
    /**
     * Add a new equality as a pair of strings to the model
     * 
     * @return true iff the strings have been parsed successfully
     */
    public boolean addRow ( String p_left, String p_right ) {
	Term left, right;
        try {
            left = parseTerm ( p_left );
        } catch ( ParserException e ) {
            setErrorStatus ( e, "left" );
            Logger.getLogger ( UserConstraintView.class.getName () ).info ( e );
            displayError ( e, newEqLeftText );
            return false;
        }

	try {
            right = parseTerm ( p_right );
	} catch ( ParserException e ) {
            setErrorStatus ( e, "right" );
            Logger.getLogger ( UserConstraintView.class.getName () ).info ( e );
            displayError ( e, newEqRightText );
            return false;
        }

	// This is only valid for sort hierarchies that are trees
	if ( !( left .sort ().extendsTrans ( right.sort () ) ||
		right.sort ().extendsTrans ( left .sort () ) ) ) {
	    printStatus ( "Sorts are incompatible" );
	    return false;
	}
	
	if ( !Constraint.BOTTOM.unify ( left, right ).isSatisfiable () ) {
	    printStatus ( "Terms are not unifiable" );
	    return false;
	}
	

	constraintTableModel.addEquality ( left, right );

	printStatus ( "Added Constraint" );

	return true;
    }    


    private void setErrorStatus (ParserException p_e, String p_leftRight) {
        printStatus ( "<html>Syntax error in " + p_leftRight + " term:<pre>"
                      + p_e.getMessage () + "</pre></html>" );
    }


    /**
     * Make one of the text fields react to a syntax error that occurred when
     * parsing the displayed string
     */
    private static void displayError (ParserException p_ex,
                                      JTextField p_textarea) {
        p_textarea.requestFocus ();
        if ( p_ex.getLocation () != null )
            p_textarea.setCaretPosition ( Math.min ( p_ex.getLocation ().getColumn (),
                                                     p_textarea.getColumns () ) );
    }


    private class SelectionListener implements KeYSelectionListener {

	/** focused node has changed */
	public void selectedNodeChanged(KeYSelectionEvent e) {}

	/** the selected proof has changed (e.g. a new proof has been
	 * loaded) */ 
	public void selectedProofChanged(KeYSelectionEvent e) {
	    final Proof selectedProof = e.getSource().getSelectedProof();
            Runnable action = new Runnable () {
		public void run () {
                    if (selectedProof != null) {
		        setConstraintTableModel(
                            selectedProof.getUserConstraint());
                    } else {
                        setConstraintTableModel(new ConstraintTableModel());
//                        constraintTable.setModel(null);
                    }
		}
	    };
        
	    if (SwingUtilities.isEventDispatchThread())  
                action.run();
            else  SwingUtilities.invokeLater(action);
	}
	
    }


    private class ConstraintTableListener implements TableModelListener {

	public void tableChanged ( TableModelEvent p_e ) {
            Runnable action = new Runnable () {
		public void run () {
		    printSatisfiability ();
		}
	    };
	    if (SwingUtilities.isEventDispatchThread())  
                action.run();
            else  SwingUtilities.invokeLater(action);
	}

    }


    private class ConstraintViewGUIListener implements GUIListener {
	/** invoked if a frame that wants modal access is opened */
	public void modalDialogOpened(GUIEvent e) {
	    setControlsActive ( false );
	}

	/** invoked if a frame that wants modal access is closed */
	public void modalDialogClosed(GUIEvent e) {
	    setControlsActive ( true );
	}

	public void shutDown(GUIEvent e) {

	}	
    }

}
