// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.collection.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import java.io.*;
import de.uka.ilkd.key.gui.Main;
import java.util.Vector;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.Services;


/** represents an input from a .key user problem file producing environment
 *  as well as a proof obligation.
 */
public class KeYUserProblemFile extends KeYFile implements ProofOblInput{

    private Term problemTerm = null;
    private String problemHeader = "";


    /** creates a new representation of a KeYSortFile with the given name,
     * a rule source representing the physical source of the input, and
     * a graphical representation to call back in order to report the progress
     * while reading.
     */
    public KeYUserProblemFile(String name, File file, Main main) {
	super(name, file, main);
    }


    /** reads the whole .key file and modifies the initial configuration
     * assigned to this object according to the given modification strategy. 
     * Throws an exception if no initial configuration has been set yet.
     */
    public void read(ModStrategy mod) throws ProofInputException {
	if (initConfig==null) {
	    throw new IllegalStateException("KeYFile: InitConfig not set.");
	}
	try {
	    CountingBufferedInputStream cinp = 
		new CountingBufferedInputStream
		    (getNewStream(),main,getNumberOfChars()/100);
            DeclPicker lexer = new DeclPicker(new KeYLexer(cinp));

	    /*
	      two namespace sets which share all namespace except the
	      variable namespace 	     	      
	    */
	    NamespaceSet normal = initConfig.namespaces().copy();
	    NamespaceSet schema = 
		new NamespaceSet(normal.variables().copy(), 
				 normal.functions(),
				 normal.sorts(), 
				 normal.heuristics(),
				 normal.choices(),
				 normal.programVariables());
	    Choice defaultChoice = initConfig.getDefaultChoice().copy();
	    ParserConfig normalConfig 
		= new ParserConfig(initConfig.getServices(),
				   normal, defaultChoice);

	    ParserConfig schemaConfig 
		= new ParserConfig(initConfig.getServices(), 
				   schema, defaultChoice);
	    problemParser = new ProblemParser(lexer, 
					      file.toString(), 
					      schemaConfig, 
					      normalConfig,
					      initConfig.getTaclet2Builder(),
					      initConfig.getTaclets()); 
            problemTerm = problemParser.problem();
	    initConfig.setTaclets(problemParser.getTaclets());
	    initConfig.add(normalConfig.namespaces(), mod);
	    initConfig.setDefaultChoice(normalConfig.getDefaultChoice()); 
            problemHeader = lexer.getText();
            if (problemHeader == null) throw new ProofInputException(
               "No problem defined in file.");
            problemHeader = 
               problemHeader.substring(0, problemHeader.lastIndexOf("problem"));
	} catch (antlr.ANTLRException e) {
	    throw new ProofInputException(e);
        }
    }

    public ProofList getPO() {
        if (problemTerm == null) throw new IllegalStateException(
           "KeYUserProblemFile: getPO() without read()");
	return new ProofList(
            new Proof(name(), problemTerm, problemHeader,
                      initConfig.createTacletIndex(), 
		      initConfig.createBuiltInRuleIndex(),
		      initConfig.getActivatedChoices()), name());
    }

    /** returns the <code>java.io.file</code> file encapsulated by
     * the <code>KeYUserProblemFile</code>.
     */
    public File getFile(){
	return file.file();
    }
    
    /**
     * @return Returns the problemHeader.
     */
    protected String getProblemHeader () {
        return problemHeader;
    }
}
